# SPDX-License-Identifier: Apache-2.0
# Licensed to the Ed-Fi Alliance under one or more agreements.
# The Ed-Fi Alliance licenses this file to you under the Apache License, Version 2.0.
# See the LICENSE and NOTICES files in the project root for more information.

import logging
from edfi_lms_harmonizer.migrator import migrate
from edfi_lms_harmonizer.helpers.argparser import MainArguments
from edfi_lms_harmonizer.exceptions_reports import (
    create_exception_reports,
    print_summary,
)
from edfi_lms_harmonizer.harmonizer import (
    harmonize_users,
    harmonize_sections,
    harmonize_assignments,
    harmonize_assignment_submissions
)


logger = logging.getLogger(__name__)

#
# Developer note: this class will not be unit tested
#


def run(arguments: MainArguments) -> None:
    """
    Orchestrates the primary business functions of LMS Harmonizer

    Parameters
    ----------
    arguments: MainArguments
        An object containing the system arguments
    """

    logger.info("Starting the Ed-Fi LMS Harmonizer")

    try:
        adapter = arguments.get_adapter()
        migrate(arguments.engine, adapter)

        exceptions_report_directory = arguments.exceptions_report_directory

        harmonize_users(arguments.engine, adapter)
        harmonize_sections(arguments.engine, adapter)
        harmonize_assignments(arguments.engine, adapter)
        harmonize_assignment_submissions(arguments.engine, adapter)

        if exceptions_report_directory is not None:
            create_exception_reports(arguments.engine, adapter, exceptions_report_directory)

        print_summary(adapter)

        logger.info("Finishing the Ed-Fi LMS Harmonizer")
    finally:
        arguments.get_adapter().engine.dispose()



# SPDX-License-Identifier: Apache-2.0
# Licensed to the Ed-Fi Alliance under one or more agreements.
# The Ed-Fi Alliance licenses this file to you under the Apache License, Version 2.0.
# See the LICENSE and NOTICES files in the project root for more information.

from datetime import datetime
from logging import getLogger
from os import path, makedirs

import pandas as pd

from edfi_lms_extractor_lib.helpers.decorators import catch_exceptions
from edfi_sql_adapter.sql_adapter import Adapter
from edfi_lms_harmonizer.sql_for_exceptions_report import (
    QUERY_FOR_ASSIGNMENT_CAT_DESCRIPTORS_MSSQL,
    QUERY_FOR_ASSIGNMENT_CAT_DESCRIPTORS_POSTGRESQL,
    QUERY_FOR_ASSIGNMENT_CAT_DESCRIPTORS_SUMMARY,
    QUERY_FOR_ASSIGNMENT_SUBMISSION_EXCEPTIONS,
    QUERY_FOR_ASSIGNMENT_EXCEPTIONS,
    QUERY_FOR_SECTION_SUMMARY,
    QUERY_FOR_SECTIONS,
    QUERY_FOR_SUBMISSION_STATUS_DESCRIPTORS_MSSQL,
    QUERY_FOR_SUBMISSION_STATUS_DESCRIPTORS_POSTGRESQL,
    QUERY_FOR_SUBMISSION_STATUS_DESCRIPTORS_SUMMARY,
    QUERY_FOR_USERS,
    QUERY_FOR_USERS_SUMMARY,
)
from edfi_lms_harmonizer.helpers.constants import DB_ENGINE


SECTIONS = "sections"
USERS = "users"
DESCRIPTORS = "descriptors"

logger = getLogger(__name__)


def _get_file_path(output_directory: str, report_type: str) -> str:
    dir_path = path.join(output_directory, report_type)

    makedirs(dir_path, exist_ok=True)

    file_path = path.join(
        dir_path, f"{datetime.now().strftime('%Y-%m-%d-%H-%M-%S')}.csv"
    )

    return file_path


def _print_summary_for_sections_and_users(adapter: Adapter) -> None:
    users_count = adapter.get_int(QUERY_FOR_USERS_SUMMARY)
    sections_count = adapter.get_int(QUERY_FOR_SECTION_SUMMARY)
    if sections_count > 0 or users_count > 0:
        logger.warning(
            f"There are {sections_count} unmatched sections and {users_count} "
            f"unmatched users in the database."
        )
    else:
        logger.debug("There are no unmatched sections or users in the database.")


def _print_summary_for_assignments_and_submissions(adapter: Adapter) -> None:
    assignments_count = adapter.get_int(QUERY_FOR_ASSIGNMENT_EXCEPTIONS)
    submissions_count = adapter.get_int(QUERY_FOR_ASSIGNMENT_SUBMISSION_EXCEPTIONS)
    if assignments_count > 0 or submissions_count > 0:
        logger.warning(
            f"There are {assignments_count} unmatched Assignments and"
            f" {submissions_count} unmatched Submissions"
        )
    else:
        logger.debug(
            "There are no unmatched Assignments or Assignment submissions in the database."
        )


def _print_summary_for_descriptors(adapter: Adapter) -> None:
    assignment_descriptors_count = adapter.get_int(
        QUERY_FOR_ASSIGNMENT_CAT_DESCRIPTORS_SUMMARY
    )
    submission_descriptors_count = adapter.get_int(
        QUERY_FOR_SUBMISSION_STATUS_DESCRIPTORS_SUMMARY
    )
    if assignment_descriptors_count > 0 or submission_descriptors_count > 0:
        logger.warning(
            f"There are {assignment_descriptors_count} missing descriptors for Assignment Category "
            f"and {submission_descriptors_count} missing descriptors for Submission Status"
        )
    else:
        logger.debug(
            "There are no missing descriptors for Assignment Category or Submission Status in the database."
        )


@catch_exceptions
def print_summary(adapter: Adapter) -> None:
    _print_summary_for_sections_and_users(adapter)
    _print_summary_for_assignments_and_submissions(adapter)
    _print_summary_for_descriptors(adapter)


@catch_exceptions
def create_exception_reports(
    engine: str, adapter: Adapter, output_directory: str
) -> None:
    logger.info("Writing the Sections exception report")
    sections = pd.read_sql(QUERY_FOR_SECTIONS, adapter.engine)
    sections.to_csv(_get_file_path(output_directory, SECTIONS), index=False)

    logger.info("Writing the Users exception report")
    users = pd.read_sql(QUERY_FOR_USERS, adapter.engine)
    users.to_csv(_get_file_path(output_directory, USERS), index=False)

    logger.info(
        "Writing the Assignment Category and Submission Status missing descriptors report"
    )

    assignment_cat_sql: str
    submission_status_sql: str
    if engine == DB_ENGINE.MSSQL:
        assignment_cat_sql = QUERY_FOR_ASSIGNMENT_CAT_DESCRIPTORS_MSSQL
        submission_status_sql = QUERY_FOR_SUBMISSION_STATUS_DESCRIPTORS_MSSQL
    else:
        assignment_cat_sql = QUERY_FOR_ASSIGNMENT_CAT_DESCRIPTORS_POSTGRESQL
        submission_status_sql = QUERY_FOR_SUBMISSION_STATUS_DESCRIPTORS_POSTGRESQL

    assignment_cat = pd.read_sql(assignment_cat_sql, adapter.engine)
    submission_status = pd.read_sql(submission_status_sql, adapter.engine)
    descriptors_report = pd.concat([assignment_cat, submission_status])
    descriptors_report.to_csv(
        _get_file_path(output_directory, DESCRIPTORS), index=False
    )



# SPDX-License-Identifier: Apache-2.0
# Licensed to the Ed-Fi Alliance under one or more agreements.
# The Ed-Fi Alliance licenses this file to you under the Apache License, Version 2.0.
# See the LICENSE and NOTICES files in the project root for more information.

import logging
import sys

from dotenv import load_dotenv
from errorhandler import ErrorHandler  # type: ignore

from edfi_lms_extractor_lib.helpers.decorators import catch_exceptions
from edfi_lms_harmonizer.helpers.argparser import MainArguments, parse_main_arguments  # type: ignore
from edfi_lms_harmonizer.runner import run

logger: logging.Logger


@catch_exceptions
def _configure_logging(arguments: MainArguments) -> None:
    global logger

    logger = logging.getLogger(__name__)

    logging.basicConfig(
        handlers=[
            logging.StreamHandler(sys.stdout),
        ],
        format="%(asctime)s - %(levelname)s - %(name)s - %(message)s",
        level=arguments.log_level,
    )


def main() -> None:
    global logger

    load_dotenv()
    arguments = parse_main_arguments(sys.argv[1:])
    _configure_logging(arguments)
    error_tracker: ErrorHandler = ErrorHandler()

    try:
        run(arguments)
    except Exception as error:
        logger.error(error)
        print(error.__traceback__, file=sys.stderr)

    if error_tracker.fired:
        print(
            "A fatal error occurred, please review the log output for more information.",
            file=sys.stderr,
        )
        sys.exit(1)
    sys.exit(0)


if __name__ == "__main__":
    main()



# SPDX-License-Identifier: Apache-2.0
# Licensed to the Ed-Fi Alliance under one or more agreements.
# The Ed-Fi Alliance licenses this file to you under the Apache License, Version 2.0.
# See the LICENSE and NOTICES files in the project root for more information.

import logging
from os import path, scandir
from typing import List

from sqlparse import split

from edfi_sql_adapter.sql_adapter import Adapter, Statement


logger = logging.getLogger(__name__)


SCHEMA_SCRIPT_NAME = "0001_initialize_lms_database.sql"


def _get_scripts_dir(engine: str) -> str:
    return path.join(path.dirname(__file__), "scripts", engine)


def _get_scripts(scripts_dir: str) -> List[str]:
    with scandir(scripts_dir) as entries:
        return sorted(
            [
                entry.path
                for entry in entries
                if entry.is_file() and entry.name.endswith(".sql") and not entry.name == SCHEMA_SCRIPT_NAME
            ]
        )


def _read_statements_from_file(full_path: str) -> List[str]:
    raw_sql: str
    with open(full_path) as f:
        raw_sql = f.read()

    statements: List[str] = split(raw_sql)
    return statements


def _migration_name(migration: str) -> str:
    return path.split(migration)[-1].split(".")[0].lower()


def _script_has_been_run(adapter: Adapter, migration: str) -> bool:
    migration_name = _migration_name(migration)
    statement = f"select 1 from lms.migrationjournal_harmonizer where script = '{migration_name}';"
    response = adapter.get_int(statement)

    return bool(response == 1)


def _record_migration_in_journal(adapter: Adapter, migration: str) -> None:
    migration_name = _migration_name(migration)

    statement = Statement(
        f"insert into lms.migrationjournal_harmonizer (script) values ('{migration_name}');",
        "Updating migration journal table",
    )

    adapter.execute([statement])


def _lms_migration_journal_exists(adapter: Adapter) -> bool:
    statement = """
select
    count(table_name)
from
    information_schema.tables
where
    table_schema like 'lms' and
    -- First option covers SQL Server if set to case sensitive mode,
    -- second option covers PostgreSQL
    table_name in ('MigrationJournal_Harmonizer', 'migrationjournal_harmonizer');
"""
    response = adapter.get_int(statement)
    return bool(response == 1)


def _run_migration_script(adapter: Adapter, migration: str) -> None:

    logger.debug(f"Running migration {migration}...")

    statements = _read_statements_from_file(migration)
    adapter.execute_script(statements)

    _record_migration_in_journal(adapter, migration)

    logger.debug(f"Done with migration {migration}.")


def _initialize_migrations(scripts_dir: str, adapter: Adapter):
    logger.debug("Creating schema and migration table...")

    migration = path.join(scripts_dir, SCHEMA_SCRIPT_NAME)
    _run_migration_script(adapter, migration)


def migrate(engine: str, adapter: Adapter) -> None:
    """
    Runs database migration scripts for installing LMS table schema into the
    destination database.

    Parameters
    ----------
    adapter: sql_adapter
        SQL Alchemy database engine object.
    """
    logger.info("Begin database auto-migration...")

    scripts_dir = _get_scripts_dir(engine)

    if not _lms_migration_journal_exists(adapter):
        _initialize_migrations(scripts_dir, adapter)

    for migration in _get_scripts(scripts_dir):
        # The following block of code does not belong in _run_migration_script
        # because it will throw an exception if the migration journal does not
        # exist, and therefore is not appropriate when initializing the LMS
        # database.
        if _script_has_been_run(adapter, migration):
            logger.debug(
                f"Migration {migration} has already run and will not be re-run."
            )
            continue

        _run_migration_script(adapter, migration)

    logger.info("Done with database auto-migration.")



# SPDX-License-Identifier: Apache-2.0
# Licensed to the Ed-Fi Alliance under one or more agreements.
# The Ed-Fi Alliance licenses this file to you under the Apache License, Version 2.0.
# See the LICENSE and NOTICES files in the project root for more information.

LOG_LEVELS = ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]


class SOURCE_SYSTEM:
    GOOGLE = "Google"
    SCHOOLOGY = "Schoology"
    CANVAS = "Canvas"


class SOURCE_SYSTEM_NAMESPACE:
    GOOGLE = "uri://google.com/classroom"
    SCHOOLOGY = "uri://schoology.com"
    CANVAS = "uri://instructure.com/canvas"


class DB_ENGINE:
    MSSQL = "mssql"
    POSTGRESQL = "postgresql"



# SPDX-License-Identifier: Apache-2.0
# Licensed to the Ed-Fi Alliance under one or more agreements.
# The Ed-Fi Alliance licenses this file to you under the Apache License, Version 2.0.
# See the LICENSE and NOTICES files in the project root for more information.

import logging

from edfi_lms_extractor_lib.helpers.decorators import catch_exceptions
from edfi_sql_adapter.sql_adapter import Adapter, Statement
from edfi_lms_harmonizer.helpers.constants import (
    SOURCE_SYSTEM,
    SOURCE_SYSTEM_NAMESPACE,
    DB_ENGINE,
)


logger = logging.getLogger(__name__)


def _generate_call_to_stored_procedure(
    engine: str, procedure: str, namespace: str = None, source_system: str = None
) -> str:
    if engine == DB_ENGINE.MSSQL:
        statement = f"EXEC lms.{procedure}"
        if namespace is not None or source_system is not None:
            statement += f" @Namespace='{namespace}', @SourceSystem='{source_system}'"
        return statement

    statement = f"call lms.{procedure}("
    if namespace is not None and source_system is not None:
        statement += f" _namespace => '{namespace}',  _sourcesystem => '{source_system}'"
    return statement + ")"


@catch_exceptions
def harmonize_users(engine: str, adapter: Adapter) -> None:

    statements = [
        Statement(
            _generate_call_to_stored_procedure(engine, "harmonize_lmsuser_canvas"),
            "Harmonizing Canvas LMS Users.",
        ),
        Statement(
            _generate_call_to_stored_procedure(
                engine, "harmonize_lmsuser_google_classroom"
            ),
            "Harmonizing Google Classroom LMS Users.",
        ),
        Statement(
            _generate_call_to_stored_procedure(engine, "harmonize_lmsuser_schoology"),
            "Harmonizing Schoology LMS Users.",
        ),
    ]

    adapter.execute(statements)


@catch_exceptions
def harmonize_sections(engine: str, adapter: Adapter) -> None:

    statements = [
        Statement(
            _generate_call_to_stored_procedure(engine, "harmonize_lmssection_canvas"),
            "Harmonizing Canvas LMS Sections.",
        ),
        Statement(
            _generate_call_to_stored_procedure(
                engine, "harmonize_lmssection_google_classroom"
            ),
            "Harmonizing Google Classroom LMS Sections.",
        ),
        Statement(
            _generate_call_to_stored_procedure(
                engine, "harmonize_lmssection_schoology"
            ),
            "Harmonizing Schoology LMS Sections.",
        ),
    ]

    adapter.execute(statements)


@catch_exceptions
def harmonize_assignments(engine: str, adapter: Adapter) -> None:

    statements = [
        Statement(
            _generate_call_to_stored_procedure(
                engine,
                "harmonize_assignment",
                SOURCE_SYSTEM_NAMESPACE.CANVAS,
                SOURCE_SYSTEM.CANVAS,
            ),
            "Harmonizing Canvas LMS Assignments.",
        ),
        Statement(
            _generate_call_to_stored_procedure(
                engine,
                "harmonize_assignment",
                SOURCE_SYSTEM_NAMESPACE.GOOGLE,
                SOURCE_SYSTEM.GOOGLE,
            ),
            "Harmonizing Google Classroom LMS Assignments.",
        ),
        Statement(
            _generate_call_to_stored_procedure(
                engine,
                "harmonize_assignment",
                SOURCE_SYSTEM_NAMESPACE.SCHOOLOGY,
                SOURCE_SYSTEM.SCHOOLOGY
            ),
            "Harmonizing Schoology LMS Assignments.",
        ),
    ]

    adapter.execute(statements)


@catch_exceptions
def harmonize_assignment_submissions(engine: str, adapter: Adapter) -> None:

    statements = [
        Statement(
            _generate_call_to_stored_procedure(
                engine,
                "harmonize_assignment_submissions",
                SOURCE_SYSTEM_NAMESPACE.CANVAS,
                SOURCE_SYSTEM.CANVAS,
            ),
            "Harmonizing Canvas LMS Assignment Submissions.",
        ),
        Statement(
            _generate_call_to_stored_procedure(
                engine,
                "harmonize_assignment_submissions",
                SOURCE_SYSTEM_NAMESPACE.GOOGLE,
                SOURCE_SYSTEM.GOOGLE,
            ),
            "Harmonizing Google Classroom LMS Assignment Submissions.",
        ),
        Statement(
            _generate_call_to_stored_procedure(
                engine,
                "harmonize_assignment_submissions",
                SOURCE_SYSTEM_NAMESPACE.SCHOOLOGY,
                SOURCE_SYSTEM.SCHOOLOGY,
            ),
            "Harmonizing Schoology LMS Assignment Submissions.",
        ),
    ]

    adapter.execute(statements)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['edfi_lms_harmonizer', 'edfi_lms_harmonizer.helpers']

package_data = \
{'': ['*'], 'edfi_lms_harmonizer': ['scripts/mssql/*', 'scripts/postgresql/*']}

install_requires = \
['ConfigArgParse>=1.4.1,<2.0.0',
 'SQLAlchemy==1.3.23',
 'edfi-lms-extractor-lib>=1.1.0,<2.0.0',
 'edfi-sql-adapter>=1.0.0,<2.0.0',
 'errorhandler>=2.0.1,<3.0.0',
 'pandas>=1.2.4,<2.0.0',
 'pyodbc>=4.0.30,<5.0.0',
 'python-dotenv>=0.17.1,<0.18.0',
 'sqlparse>=0.4.1,<0.5.0']

setup_kwargs = {
    'name': 'edfi-lms-harmonizer',
    'version': '1.2.0',
    'description': 'Ed-Fi LMS Toolkit Harmonizer',
    'long_description': '# lms-harmonizer\n\nUtility / process for linking LMS data with SIS data in an Ed-Fi ODS. Assumes\nthat LMS data have been retrieved from an LMS via an LMS Extractors and uploaded\ninto tables in the lms schema via LMS Data Store Loader.\n\n## Harmonization Process\n\nThe primary duty of the LMS Harmonizer is to match Student and Sections found in\nthe data extracted from upstream Learning Management Systems (LMS) with the same\nentities in an Ed-Fi ODS database, which are sourced from a Student Information\nSystem (SIS). These systems often do not have perfect alignment, so that the\n"harmonization" process used by this tool is imperfect and may need\ncustomization to fit your implementation.\n\nAny LMS records that cannot be matched to SIS records are considered\n"exceptions"; you can generate CSV exports listing the exceptions by using one\nof the command line options described below. With that information, you can then\neither update the LMS source system by providing it with additional information,\nor you can customize this process.\n\nThe out-of-the box solution has the following mapping logic:\n\n### Student Mapping\n\nFor Canvas and Schoology, the default assumption is that the SIS\'s unique\nidentifier for the student, which has been loaded into the\n`Student.StudentUniqueId` field via the Ed-FI ODS / PAI, has also been loaded\ninto the LMS in the following field:\n\n* Canvas: `sis_user_id` ([API documentation](https://canvas.instructure.com/doc/api/users.html#User))\n* Schoology: `school_uid` ([API documentation](https://developers.schoology.com/api-documentation/rest-api-v1/user))\n\nGoogle Classroom does not have such a field. Instead, the Harmonizer assumes\nthat the student\'s email address used in Google Classroom is also recorded in\nthe SIS and loaded into the ODS. The Harmonizer then does a simple matching of\nrecords by that email address, under the assumption that no two students have\nthe same email address.\n\n### Section Mapping\n\nThe solution currently assumes that the SIS has a globally-unique identifier,\nwhich has been loaded into the `Section.SectionIdentifier` field in the ODS /\nAPI. This same value is then assumed to be loaded into the following field for each LMS:\n\n* Canvas: `sis_section_id` on a course ([API\n  documentation](https://canvas.instructure.com/doc/api/courses.html))\n* Google Classroom: `course.aliases` object ([API\n  documentation](https://developers.google.com/classroom/reference/rest/v1/courses.aliases))\n* Schoology: `section_school_code` on a course section ([API\n  documentation](https://developers.schoology.com/api-documentation/rest-api-v1/course-section))\n\n### Customization\n\nIf your implementation does not match the logic described above, then you can clone\nthis repository and modify the stored procedures to fit alternate logic.\n\n## Getting Started\n\n1. Requires Python 3.9+ and Poetry.\n1. Requires that you have the ODS/API Suite 3, Version 5.2 or Version 5.3 and have [installed the\n   Ed-Fi-LMS extension](../../docs/installing-the-extension.md).\n1. Install required Python packages:\n\n   ```bash\n   poetry install\n   ```\n\n1. Create a `.env` file, or plan to pass all [configuration](#configuration) via\n   command line arguments. See [.env.example](.env.example).\n\n### Configuration\n\nSupported parameters:\n\n| Description                 | Required            | Command Line Argument                   | Environment Variable        |\n| --------------------------- | ------------------- | --------------------------------------- | --------------------------- |\n| DB Engine                   | no                  | `-g` or `--engine` (mssql | postgresql) | DB_ENGINE                   |\n| DB Server                   | yes                 | `-s` or `--server`                      | DB_SERVER                   |\n| DB Port                     | no (default: 1433)  | `--port`                                | DB_PORT                     |\n| DB Name                     | yes                 | `-d` or `--dbname`                      | DB_NAME                     |\n| Exceptions report directory | no (no default)     | `-e` or `--exceptions-report-directory` | EXCEPTIONS_REPORT_DIRECTORY |\n| DB Username **              | no (no default)     | `-u` or `--username`                    | DB_USERNAME                 |\n| DB Password **              | no (no default)     | `-p` or `--password`                    | DB_PASSWORD                 |\n| Use integrated security **  | no (default: false) | `-i` or `--useintegratedsecurity`       | USE_INTEGRATED_SECURITY     |\n| Log level*                  | no (default: INFO)  | `-l` or `--log-level`                   | LOG_LEVEL                   |\n| Encrypt db connection       | no (default: False) | `-n` or `--encrypt`                     | ENCRYPT_SQL_CONNECTION      |\n| Trust db server certificate | no (default: False) | `-t` or `--trust-certificate`           | TRUST_SERVER_CERTIFICATE    |\n\n\\* Valid values for the optional _log level_:\n\n* DEBUG\n* INFO(default)\n* WARNING\n* ERROR\n* CRITICAL\n\n\\** If using integrated security, DB Username and password won\'t be required,\notherwise they are required.\n\n### Running the Tool\n\nFor detailed help, execute `poetry run python edfi_lms_harmonizer -h`.\n\nSample call using full integrated security, loading from the sample files\ndirectory:\n\n```bash\npoetry run python edfi_lms_harmonizer --server localhost --dbname lms_toolkit --useintegratedsecurity\n```\n\n## Developer Notes\n\n### Dev Operations\n\n1. Style check: `poetry run flake8`\n1. Static typing check: `poetry run mypy .`\n1. Run unit tests: `poetry run pytest tests`\n1. Run unit tests with code coverage: `poetry run coverage run -m pytest tests`\n1. View code coverage: `poetry run coverage report`\n1. Run SQL Server integration tests: `poetry run pytest tests_integration_mssql`\n1. Run PostgreSQL integration tests: `poetry run pytest tests_integration_pgsql`\n\n_Also see\n[build.py](https://github.com/Ed-Fi-Exchange-OSS/LMS-Toolkit/blob/main/docs/build.md)_\nfor use of the build script.\n\n### Integration Testing\n\nSee Integration test setup instructions ([SQL\nServer](./tests_integration_mssql/README.md) |\n[PostgreSQL](./tests_integration_pgsql/README.md)) for information on\nconfiguring integration testing to work in various environments.\n\n## Legal Information\n\nCopyright (c) 2022 Ed-Fi Alliance, LLC and contributors.\n\nLicensed under the [Apache License, Version\n2.0](https://github.com/Ed-Fi-Exchange-OSS/LMS-Toolkit/blob/main/LICENSE) (the\n"License").\n\nUnless required by applicable law or agreed to in writing, software distributed\nunder the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR\nCONDITIONS OF ANY KIND, either express or implied. See the License for the\nspecific language governing permissions and limitations under the License.\n\nSee\n[NOTICES](https://github.com/Ed-Fi-Exchange-OSS/LMS-Toolkit/blob/main/NOTICES.md)\nfor additional copyright and license notifications.\n',
    'author': 'Ed-Fi Alliance, LLC and contributors',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://techdocs.ed-fi.org/display/EDFITOOLS/LMS+Toolkit',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



# SPDX-License-Identifier: Apache-2.0
# Licensed to the Ed-Fi Alliance under one or more agreements.
# The Ed-Fi Alliance licenses this file to you under the Apache License, Version 2.0.
# See the LICENSE and NOTICES files in the project root for more information.

from dataclasses import dataclass
import os
from typing import List, Optional

from configargparse import ArgParser  # type: ignore

from .constants import LOG_LEVELS, DB_ENGINE
from edfi_sql_adapter import sql_adapter


@dataclass
class MainArguments:
    """
    Container for holding arguments parsed at the command line.

    Parameters
    ----------
    log_level : str
        The log level for the tool.
    """

    log_level: str
    exceptions_report_directory: Optional[str]
    engine: str
    server: str
    db_name: str
    username: str
    password: str
    port: int
    encrypt: bool = False
    trust_certificate: bool = False
    useintegratedsecurity: bool = False

    def __post_init__(self) -> None:
        self.adapter: sql_adapter.Adapter

    def _get_sql_server_port(self) -> int:
        return 1433 if not self.port or self.port == 0 else self.port

    def _get_pgsql_server_port(self) -> int:
        return 5432 if not self.port or self.port == 0 else self.port

    def _assert_credentials(self, username: str, password: str) -> None:
        if username is None:
            raise RuntimeError("No database username was provided.")
        if password is None:
            raise RuntimeError("No database password was provided.")

    def build_mssql_adapter(self) -> None:
        self._assert_credentials(self.username, self.password)

        self.adapter = sql_adapter.create_mssql_adapter(
            self.username,
            self.password,
            self.server,
            self.db_name,
            self._get_sql_server_port(),
            self.encrypt,
            self.trust_certificate,
        )

    def build_mssql_adapter_with_integrated_security(self) -> None:
        self.adapter = sql_adapter.create_mssql_adapter_with_integrated_security(
            self.server,
            self.db_name,
            self._get_sql_server_port(),
            self.encrypt,
            self.trust_certificate,
        )

    def build_postgresql_adapter(self) -> None:
        self._assert_credentials(self.username, self.password)

        self.adapter = sql_adapter.create_postgresql_adapter(
            self.username, self.password, self.server, self.db_name, self._get_pgsql_server_port()
        )

    def get_adapter(self) -> sql_adapter.Adapter:
        if not hasattr(self, 'adapter'):
            if self.engine == DB_ENGINE.MSSQL:
                if self.useintegratedsecurity:
                    self.build_mssql_adapter_with_integrated_security()
                else:
                    self.build_mssql_adapter()
            else:
                self.build_postgresql_adapter()

        return self.adapter


def parse_main_arguments(args_in: List[str]) -> MainArguments:
    """
    Configures the command-line interface.

    Parameters
    ----------
    args_in : list of str
        Full argument list from the command line.

    Returns
    -------
    arguments  : MainArguments
        A populated `MainArguments` object.
    """

    parser = ArgParser()

    parser.add(  # type: ignore
        "-s",
        "--server",
        help="Database server name or IP address",
        required=True,
        env_var="DB_SERVER",
    )
    parser.add(  # type: ignore
        "--port",
        help="Database server port number",
        type=int,
        env_var="DB_PORT",
        default=1433,
    )
    parser.add(  # type: ignore
        "-d",
        "--dbname",
        help="Name of the database with the LMS tables.",
        env_var="DB_NAME",
        required=True,
    )

    USE_INTEGRATED = "--useintegratedsecurity"
    USE_INTEGRATED_SHORT = "-i"

    parser.add(  # type: ignore
        USE_INTEGRATED_SHORT,
        USE_INTEGRATED,
        help="Use Integrated Security for the database connection.",
        action="store_true",
    )
    user_name_required = (
        USE_INTEGRATED not in args_in and USE_INTEGRATED_SHORT not in args_in
    )

    # Retrieve this value because we need it in order to determine
    # if username and password are required
    integrated_env_var = os.getenv("USE_INTEGRATED_SECURITY")
    if integrated_env_var and integrated_env_var.lower() in ("true", "yes", "t", "y"):
        user_name_required = False

    parser.add(  # type: ignore
        "-u",
        "--username",
        required=user_name_required,
        env_var="DB_USERNAME",
        help="Database username, when not using integrated security.",
    )
    parser.add(  # type: ignore
        "-p",
        "--password",
        required=user_name_required,
        env_var="DB_PASSWORD",
        help="Database user password, when not using integrated security.",
    )

    parser.add(  # type: ignore
        "-l",
        "--log-level",
        required=False,
        help="The log level for the tool.",
        choices=LOG_LEVELS,
        type=str,
        default="INFO",
        env_var="LOG_LEVEL",
    )

    parser.add(  # type: ignore
        "-e",
        "--exceptions-report-directory",
        required=False,
        help="File path for optional output of a CSV exception report.",
        type=str,
        env_var="EXCEPTIONS_REPORT_DIRECTORY",
    )

    parser.add(  # type: ignore
        "-n",
        "--encrypt",
        help="Encrypt the connection to the database.",
        action="store_true",
        env_var="ENCRYPT_SQL_CONNECTION",
    )
    parser.add(  # type: ignore
        "-t",
        "--trust-certificate",
        help="When encrypting connections, trust the server certificate. Useful for localhost debugging with a self-signed certificate. USE WITH CAUTION.",
        action="store_true",
        env_var="TRUST_SERVER_CERTIFICATE",
    )
    parser.add(  # type: ignore
        "-g",  # because 'e' and 'n' are already taken (；′⌒`)
        "--engine",
        help="Database engine: mssql for Microsoft SQL Server, or postgresql for PostgreSQL.",
        choices=[DB_ENGINE.MSSQL, DB_ENGINE.POSTGRESQL],
        default=DB_ENGINE.MSSQL,
        type=str,
        env_var="DB_ENGINE",
    )

    args_parsed = parser.parse_args(args_in)

    # Need to add this back in because reading it manually earlier
    # seems to cause it to be misread by the parser.
    args_parsed.useintegratedsecurity = (
        args_parsed.useintegratedsecurity or not user_name_required
    )

    arguments = MainArguments(
        args_parsed.log_level,
        args_parsed.exceptions_report_directory,
        args_parsed.engine,
        args_parsed.server,
        args_parsed.dbname,
        args_parsed.username,
        args_parsed.password,
        args_parsed.port,
        args_parsed.encrypt,
        args_parsed.trust_certificate,
        args_parsed.useintegratedsecurity
    )

    return arguments



# SPDX-License-Identifier: Apache-2.0
# Licensed to the Ed-Fi Alliance under one or more agreements.
# The Ed-Fi Alliance licenses this file to you under the Apache License, Version 2.0.
# See the LICENSE and NOTICES files in the project root for more information.

QUERY_FOR_SECTIONS = "select * from lmsx.exceptions_lmssection"

QUERY_FOR_SECTION_SUMMARY = """
select
    count(1) as unmatchedcount
from
    lmsx.exceptions_lmssection
    """

QUERY_FOR_USERS = "select * from lmsx.exceptions_lmsuser"

QUERY_FOR_USERS_SUMMARY = """
select
    count(1) as unmatchedcount
from
    lmsx.exceptions_lmsuser
    """

QUERY_FOR_ASSIGNMENT_CAT_DESCRIPTORS_MSSQL = """
select
    distinct assignmentcategory as missingvalue,
    'assignmentcategorydescriptor' as descriptor,
    'uri://ed-fi.org/edfilms/assignmentcategorydescriptor/' + sourcesystem as namespace
from lmsx.missing_assignment_category_descriptors
"""

QUERY_FOR_ASSIGNMENT_CAT_DESCRIPTORS_POSTGRESQL = """
select
    distinct assignmentcategory as missingvalue,
    'assignmentcategorydescriptor' as descriptor,
    'uri://ed-fi.org/edfilms/assignmentcategorydescriptor/' || sourcesystem as namespace
from lmsx.missing_assignment_category_descriptors
"""

QUERY_FOR_ASSIGNMENT_CAT_DESCRIPTORS_SUMMARY = """
select
    count(distinct assignmentcategory)
from
    lmsx.missing_assignment_category_descriptors
    """

QUERY_FOR_SUBMISSION_STATUS_DESCRIPTORS_MSSQL = """
select
    distinct submissionstatus as missingvalue,
    'submissionstatusdescriptor' as descriptor,
    'uri://ed-fi.org/edfilms/submissionstatusdescriptor/' + sourcesystem as namespace
from lmsx.missing_assignment_submission_status_descriptors
"""

QUERY_FOR_SUBMISSION_STATUS_DESCRIPTORS_POSTGRESQL = """
select
    distinct submissionstatus as missingvalue,
    'submissionstatusdescriptor' as descriptor,
    'uri://ed-fi.org/edfilms/submissionstatusdescriptor/' || sourcesystem as namespace
from lmsx.missing_assignment_submission_status_descriptors
"""

QUERY_FOR_SUBMISSION_STATUS_DESCRIPTORS_SUMMARY = """
select
    count(distinct submissionstatus)
from
    lmsx.missing_assignment_submission_status_descriptors
    """

QUERY_FOR_ASSIGNMENT_EXCEPTIONS = """
select
    count(*)
from
    lmsx.assignments_exceptions
    """

QUERY_FOR_ASSIGNMENT_SUBMISSION_EXCEPTIONS = """
select
    count(*)
from
    lmsx.assignment_submissions_exceptions
    """
